import os
from pathlib import Path
import shutil

import filetype

from .._config import args, path
from .._config.path import RootPath
from .._core.video import Assember, DelAnd2Pickle, Video, Chapter, Course, Searcher
from .._core import vo


class Config:

    args = args
    path = path
    RootPath = RootPath
    performance = args.Performance


class VSearcher:

    config: Config = Config

    def __init__(self, url_prefix, domain_url, step, speed_x):
        # 模仿学习那种参数
        # self.config = Config()
        pass

    @classmethod
    def init(cls, domain_url,project_root_dir="", static_folder="static", output_dir="vs-output", step='fps', speed_x=1, ocr_model_dir=None):
        """
        domain_url: 外部访问静态资源时的URL前半部分
        step: 视频遍历的帧间隔, 越大速度越快
        speed_x: 对step进行整数倍的加速, 即 最终的step = step * speed_x
        ocr_model_dir: 含有.pdmodel文件的目录相对路径, 默认为: ocrv3模型
        static_folder: 可以外部url访问的静态文件夹的目录路径, 该路径是相对于项目的根路径
        output_dir: vsearch处理结果输出的保存的相对于static_folder的路径, 即: 实际的输出目录为: {static_folder}/{output_dir}
        """
        # @RISK 以下初始化顺序最好不要有改动
        cls.config.RootPath.set_project_dir(project_root_dir)
        cls.config.args.url_prefix = domain_url
        cls.set_step(step, speed_x)
        cls.set_paddle_ocr_model_dir(ocr_model_dir)
        # 如下两方法顺序不可以变
        cls.set_static_dir(static_folder) 
        cls.set_output_dir(relative_static_folder_path= output_dir)
        Assember.setOCR() # 创建OCR对象


    # @classmethod
    # def set_name_(cls, _name_):
    #     cls.config.RootPath.set_project_dir(_name_)

    @classmethod
    def set_paddle_ocr_model_dir(cls, model_dir=None):
        """
        model_dir: 'resource/model/ocr/paddle'
        default:
            rec: ch_ppocr_mobile_v2.0_rec_infer
            det: ch_ppocr_mobile_v2.0_det_infer
        可参考：https://gitee.com/paddlepaddle/PaddleOCR/blob/release/2.5/doc/doc_ch/models_list.md
        """
        cls.config.RootPath.set_model_dir(model_dir)

    def __init_path(self):
        pass

    @classmethod
    def set_output_dir(cls, relative_static_folder_path):
        cls.config.RootPath.set_output_dir(relative_static_folder_path)

    @classmethod
    def set_static_dir(cls, relative_project_dir_path):
        """ 设置URL替换前缀路径
        例如：
            project_dir: E://a/b/project_dir
            img_local_path: E://a/b/project_dir/x/ff/ss/c.png
            relative_project_dir_path: x
            url_prefix_local_path: {project_dir}/x
            url: http://localhost:5000/ff/ss/c.png
        """
        if relative_project_dir_path:
            cls.config.RootPath.set_static_folder_dir(
                relative_project_dir_path)

    @classmethod
    def set_step(cls, step, speed_x):
        """ 设置全局默认step """
        Assember.set_step(step=step, speed_x=speed_x)

    @classmethod
    def executeVideo(cls, video_file_path, video_name=None, step=None, speed_x=None, output_dir=None) -> vo.VideoVO:
        """ video_name: if None 则为 video_file_path 所示的名字 """
        if not Path(video_file_path).exists():
            print(f'路径: {video_file_path} 不存在!')
            return None
        if not filetype.is_video(video_file_path):
            print(f'传入的路径: {video_file_path} 不是视频')
            return None
        video = Assember.executeVideo(
            video_path=video_file_path, name=video_name, step=step, speed_x=speed_x, output_dir=output_dir)
        return vo.VideoVO.create(video=video)

    @classmethod
    def createCourseDir(cls, course_name):
        """ 
        创建资源对应的目录, 并返回路径
        type: course | chapter 
        return: output_dir
        """
        return Assember.createPureDir(course_name=course_name)

    @classmethod
    def createChapterDir(cls, chapter_name, course_name):
        """ 
        创建资源对应的目录, 并返回路径
        type: course | chapter 
        return: output_dir
        """
        return Assember.createPureDir(chapter_name=chapter_name, course_name=course_name)

    @classmethod
    def executeChapter(cls, chapter_dir_path, step=None, speed_x=None) -> vo.ChapterVO:
        if not Path(chapter_dir_path).exists():
            print(f'路径: {chapter_dir_path} 不存在!')
            return None
        chapter = Assember.executeChapter(
            chapter_dir_path=chapter_dir_path, step=step, speed_x=speed_x)
        return vo.ChapterVO.create(chapter)

    @classmethod
    def executeCourse(cls, course_dir_path, step=None, speed_x=None) -> vo.CourseVO:
        if not Path(course_dir_path).exists():
            print(f'路径: {course_dir_path} 不存在!')
            return None
        course = Assember.executeCourse(
            course_dir_path=course_dir_path, step=step, speed_x=speed_x)
        return vo.CourseVO.create(course)

    @classmethod
    def loadResource(cls, o_path):
        if not Path(o_path).exists():
            raise ValueError('o_path不存在')
        o = DelAnd2Pickle.loadPickle(o_path)
        if isinstance(o, Video):
            return vo.VideoVO.create(o)
        elif isinstance(o, Chapter):
            return vo.ChapterVO.create(o)
        elif isinstance(o, Course):
            return vo.CourseVO.create(o)

    @classmethod
    def releaseResource(cls, o_path):
        """ 释放o_path对应的pickle对象对应的所有存储资源 """
        o = DelAnd2Pickle.loadPickle(o_path=o_path)
        o.releaseResource()

    @classmethod
    def search(cls, o_or_path, key):
        return Searcher(o_or_path=o_or_path).search(key)

    @classmethod
    def releaseByOutputDir(cls, output_dir):
        """ 释放处理产生的文件： 1. 注释文件 2.视频文件 3.图片文件 4. 课件 @WAIT（或许实时生成比较好，就是get的时候进行获取文件） """
        if os.path.exists(output_dir):
            shutil.rmtree(output_dir, ignore_errors=True)



# ['__builtins__', '__cached__', '__doc__', '__file__', '__loader__', '__name__', '__package__', '__spec__']
import glob
from inspect import ismethod
import os
from pathlib import Path

from .args import Performance as performance_config

from . import project_root_dir

def getThisPackageRootPath():
    return ''


# 路径生成辅助函数

# def load_ocr_path(model_dir, paddle_name):
#     ''' 用于生成ocr路径 '''
#     det_path = glob.glob(f'{model_dir}\\ocr\\{paddle_name}\\*det*')
#     if len(det_path) == 0:
#         print(f'路径: {model_dir}\\ocr\\{paddle_name} |  没有det模型文件, 将使用默认的模型文件')
#         det_path = None
#     rec_path = glob.glob(f'{model_dir}\\ocr\\{paddle_name}\\*rec*')
#     if len(rec_path) == 0:
#         print( f'路径: {model_dir}\\ocr\\{paddle_name} |  没有rec模型文件, 将使用默认的模型文件' )
#         rec_path = None
#     return det_path, rec_path
#     # return None,None


class RootPath:
    """ 命名规范: 目录变量名以 "_dir"为结尾 """

    # 当前项目的根路径
    project_root_dir = project_root_dir

    # 外部可访问的静态文件夹
    static_folder_dir = str(
        Path(project_root_dir).joinpath("static"))  # 静态文件夹路径, 默认值
    static_folder_dir_prefix = str(Path(static_folder_dir).parent)  # 静态文件夹的前缀 a/c/static -> a/c
    # | 作用: 用于锁定url 例如: a/c/e.png -> url: http://xxx/a/c/e.png 但是 static_url为 http://xxx/c/e.png才能访问
    # 输出目录的路径
    output_dir = str(Path(static_folder_dir, 'vsearch-output'))

    # 搜索结果输出目录
    output_search_result_dir = str(Path(output_dir).joinpath('search-result'))


    # 处理视频后产生的关键图片保存的路径
    output_courses_dir = str(Path(output_dir).joinpath('courses'))
    output_chapters_dir = str(Path(output_dir).joinpath('chapters'))
    output_videos_dir = str(Path(output_dir).joinpath('videos'))

    # paddleocr 文字检测 和 文字分类模型 | 最新版可以自行去git的paddleocr查看, 将此处的路径末尾改名字即可自动下载
    det_model_dir, rec_model_dir = None, None  # 导入训练好的机器学习模型路径

    @classmethod
    def set_model_dir(cls, dir_path):
        """ dir_path: 项目相对路径
        case: {project_dir}/{dir_path}/**mobile**/inference.pdmodel
        """
        base_path = Path(cls.project_root_dir).joinpath(dir_path)
        result = list(base_path.glob('*det*'))
        # 判断是否真的存在模型文件
        if len(result) == 0 or len( list(result[0].glob('*.pdmodel')) ) == 0:
            print(f'路径: {dir_path} |  没有det模型文件, 将使用默认的模型文件')
        else:
            print(f'使用的检测模型为: {result[0].stem}')
            cls.det_model_dir = str(result[0])
        result = list(base_path.glob('*rec*'))
        # 判断是否真的存在模型文件
        if len(result) == 0 or len( list(result[0].glob('*.pdmodel')) ) == 0:
            print(f'路径: {dir_path} |  没有rec模型文件, 将使用默认的模型文件')
        else:
            print(f'使用的识别模型为: {result[0].stem}')
            cls.rec_model_dir = str(result[0])
        print(f'ocr_det_dir: {cls.det_model_dir}\nocr_rec_dir: {cls.rec_model_dir}')

    @classmethod
    def join_output_dir(cls, path):
        return str(Path(cls.output_dir, path))

    @classmethod
    def set_project_dir(cls, project_root_dir):
        old_project_root_dir = cls.project_root_dir
        print(f'old_project_root_dir: {old_project_root_dir}\nproject_root: {project_root_dir}')
        print("=========路径更新前========")
        for key, value in cls.__dict__.items(): 
            if "_dir" in key and value is not None \
            and key != 'project_root_dir' \
            and not isinstance(value, classmethod) \
            and not isinstance(value, staticmethod) \
            and not ismethod(value): # paddle的路径可以为None
                print(f'{key}: {value}')
                if old_project_root_dir == "" or old_project_root_dir == '.': # 原先没有值
                    setattr(cls, key, str(Path(project_root_dir, value)))
                else: # 原先不为空
                    setattr(cls, key, value.replace(old_project_root_dir, project_root_dir))
        cls.project_root_dir = project_root_dir # 完全替换
        print("=========路径更新后========")
        for key, value in cls.__dict__.items(): 
            if "_dir" in key \
            and not isinstance(value, classmethod) \
            and not isinstance(value, staticmethod) \
            and not ismethod(value): # paddle的路径可以为None
                print(f'{key}: {value}')                
        



    # | 此时就有用了
    @classmethod
    def set_output_dir(cls, relative_static_folder_path: str = ""):
        """
             {project_dir}/{static_folder_dir}/${relative_static_folder_path}
        """
        print(f'old_output_dir: {cls.output_dir}')
        if relative_static_folder_path[0] in ['/', '\\']:  # 为了避免变为F:/a/b的绝对路径
            relative_static_folder_path = relative_static_folder_path[1:]
        cls.output_dir = str(
            Path(cls.static_folder_dir, relative_static_folder_path))
        
        print(f'new_output_dir: {cls.output_dir}')
        # 与output_dir有关的所有路径都需要重置
        cls.output_courses_dir = cls.join_output_dir('courses')
        cls.output_chapters_dir = cls.join_output_dir('chapters')
        cls.output_videos_dir = cls.join_output_dir('videos')
        cls.output_search_result_dir = cls.join_output_dir('search-result')

    

    @classmethod
    def set_static_folder_dir(cls, relative_project_dir_path):
        """ 设置URL替换前缀路径
        例如：
            project_dir: E://a/b/project_dir
            img_local_path: E://a/b/project_dir/x/ff/ss/c.png
            relative_project_dir_path: x
            url_prefix_local_path: {project_dir}/x
            url: http://localhost:5000/ff/ss/c.png
        """
        old_static_folder_dir = cls.static_folder_dir
        cls.static_folder_dir = os.path.join(
            cls.project_root_dir, relative_project_dir_path)
        cls.static_folder_dir_prefix = str(Path(cls.static_folder_dir).parent)

        # 更新基于此的输出路径
        for k,v in cls.__dict__.items():
            if k.startswith('output_'):
                setattr(cls, k, v.replace(old_static_folder_dir, cls.static_folder_dir))



"""
vsearcher
------------------
"""
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="vsearcher",
    version="0.2.14",
    author="breath",
    author_email="1498408920@qq.com",
    maintainer="breath",
    description="支持视频内容检索和课件自动生成的库",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/breath57/vsearch/tree/0.2.6-pypi",
    keywords=["video", "search", "courseware", "ocr"],
    project_urls={
        # "Bug Tracker": "bug连接",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.9",
    install_requires=[
        "filetype>=1.0.13",
        "img2pdf>=0.4.4",
        "jieba>=0.42.1",
        "numpy>=1.22.4",
        "opencv_python>=4.5.5.64",
        "paddleocr>=2.5",
        "pandas>=1.4.2",
        "Pillow>=9.1.1",
        "setuptools>=58.3.1",
    ]

)
